### PersefoneDesktop - a GUI to the Persefone model of agriculture and ecosystems
###
### This file links the QML UI designs to the underlying model.
###

## For references, see:
## - https://github.com/barche/QML.jl
## - https://github.com/barche/QmlJuliaExamples
## - https://doc.qt.io/qt-6/qtquick-index.html

global model::Union{AgentBasedModel,Missing} = missing
#loadmodelobject(joinpath(dirname(@__FILE__), "../sampleinit.dat")) #initialise()
const running = Observable(false)
const date = Observable(today())
const progress = Observable(0.0)
const delay = Observable(0.5)
const runbuttontext = Observable(">>")
const runbuttontip = Observable("Run")

function newsimulation()
    global model
    running[] = false
    progress[] = 0.0
    model = initialise() #TODO configure
    date[] = model.date
    println("Model initialised.")
end

#TODO loadsimulation()
#TODO saveoutput()

function nextstep()
    global model
    if date[] < model.date
        # If we've "scrolled back" in time, step forward again without
        # having to resimulate (see `previousstep()`)
        date[] += Day(1)
    else
        # If we're already displaying the newest model update, we need to
        # keep simulating
        stepsimulation!(model)
        date[] = model.date
    end
    progress[] = (date[]-@param(core.startdate)) /
        (@param(core.enddate)-@param(core.startdate))
    println("Updated model: $(date[]-Day(1))")
end

function previousstep()
    # Since we store all model output in a dataframe, we can simply
    # "scroll back" in time and visualise the state of the model at
    # a previous date
    global model
    date[] -= Day(1)
    progress[] = (date[]-@param(core.startdate)) /
        (@param(core.enddate)-@param(core.startdate))
    println("Backtracked model: $(date[]-Day(1))")
end
    
function runsimulation()
    #XXX is this all I need to interrupt a run?
    while running[]
        nextstep()
        sleep(delay)
    end
end

function togglerunning()
    if running[]
        running[] = false
        runbuttontext[] = ">>"
        runbuttontip[] = "Run"
    else
        running[] = true
        runbuttontext[] = "||"
        runbuttontip[] = "Pause"
        runsimulation()
    end
end

function render_map(screen)
    global model
    figure = visualisemap(model, date[])
    display(screen, figure.scene)
end

function render_map(screen)
    global model
    figure = populationtrends(model)
    display(screen, figure.scene)
end

@qmlfunction newsimulation
@qmlfunction nextstep
@qmlfunction previousstep
@qmlfunction togglerunning
@qmlfunction render_map

on(delay) do d
    println("Delay is now $(round(d, digits=1)) seconds.")
end

on(running) do r
    r ? println("Simulation started.") : println("Simulation stopped.")
end

"""
    launch()

The main function that creates the application.
"""
function launch()
    # absolute path in case working dir is overridden
    qmlfile = joinpath(dirname(@__FILE__), "main.qml")
    loadqml(qmlfile,
            vars = JuliaPropertyMap("running" => running,
                                    "date" => date,
                                    "delay" => delay,
                                    "progress" => progress,
                                    "runbuttontext" => runbuttontext,
                                    "runbuttontip" => runbuttontip))
    println("Launched Persefone Desktop.")
    exec()
end
