/// PersefoneDesktop - a GUI to the Persefone model of agriculture and ecosystems
///
/// This file defines the layout of the main window.
///

import QtQuick
import QtQuick.Controls
import QtQuick.Dialogs
import QtQuick.Layouts
import org.julialang

ApplicationWindow {
	id: mainWindow
	title: "Persefone.jl Desktop"
	width: 1024
	height: 768
	visible: true
	
	menuBar: MenuBar {
		Menu {
			title: "&Simulation"
			Action {
				text: "&New Simulation"
				onTriggered: { Julia.newsimulation() } //TODO select config file
			}
			Action { text: "&Configure Simulation" }
			Action { text: "&Load Saved State" }
			Action { text: "&Save Current State" }
			Action { text: "Save Model &Output" }
			MenuSeparator { }
			Action {
				text: "&Quit"
				onTriggered: { mainWindow.close() }
			}
		}
		Menu {
			title: "&Help"
			Action {
				text: "&Documentation"
				onTriggered: { Qt.openUrlExternally("https://persefone-model.eu/documentation") }
			}
			Action {
				text: "&Website"
				onTriggered: { Qt.openUrlExternally("https://persefone-model.eu/") }
			}
			Action {
				text: "&About"
				onTriggered: { aboutDialog.open() }
			}
		}
	}
	
	MessageDialog {
		id: aboutDialog
		text: "Persefone.jl Desktop"
		informativeText: "A mechanistic model of agricultural landscapes \
and ecosystems in Europe.\n\n\
© 2023 Daniel Vedder, Lea Kolb, Guy Pe'er\n\
Distributed under the MIT license."
	}

	RowLayout {
		anchors.fill: parent

		// visualise the model map and the locations of animals
		/* MakieViewport { */
		/* 	id: mapviewport */
		/* 	Layout.fillWidth: true */
		/* 	Layout.fillHeight: true */
		/* 	renderFunction: Julia.render_map() //TODO */
		/* } */

		ColumnLayout {
			anchors.fill: parent

			// the calendar is no longer a single widget, but needs to be
			// constructed manually :-(
			ColumnLayout {
				id: calenderWidget
				Layout.alignment: Qt.AlignHCenter
				Text {
					text: "<b>"+Qt.formatDate(new Date(), "MMM yyyy")+"</b>" //FIXME
				}
				DayOfWeekRow {
					locale: grid.locale
					Layout.column: 1
					Layout.fillWidth: true
				}
				MonthGrid {
					id: grid
					//month: Calendar.December //FIXME
					//year: 2015 //FIXME
					locale: Qt.locale("en_GB")
					Layout.fillWidth: true
					Layout.fillHeight: true
					delegate: Text {
						horizontalAlignment: Text.AlignHCenter
						verticalAlignment: Text.AlignVCenter
						opacity: model.month === grid.month ? 1 : 0
						text: model.today ? "<b>"+model.day+"</b>" : model.day
						font: grid.font
						required property var model
					}
				}
			}
			
			// plot the population sizes over time
			/* MakieViewport { */
			/* 	id: plotviewport */
			/* 	Layout.fillWidth: true */
			/* 	Layout.fillHeight: true */
			/* 	renderFunction: Julia.render_plot() //TODO */
			/* } */
		}
	}

	// the main control bar, with pause/step/run buttons, the progress
	// bar and a speed slider
	footer: ToolBar {
		RowLayout {
			id: controlBar
			anchors.fill: parent
			Layout.alignment: Qt.AlignVCenter
			Layout.fillWidth: true
			// anchors.topMargin: 5 //FIXME
			// anchors.bottomMargin: 5
			Button {
				id: backButton
				text: "<"
				ToolTip.text: "Back"
				ToolTip.visible: hovered
				onClicked: { Julia.previousstep() }
			}
			Button {
				id: stepButton
				text: ">"
				ToolTip.text: "Step"
				ToolTip.visible: hovered
				onClicked: { Julia.nextstep() }
			}
			Button {
				id: runButton
				text: vars.runbuttontext
				ToolTip.text: vars.runbuttontip
				ToolTip.visible: hovered
				onClicked: { Julia.togglerunning() }
			}
			ProgressBar {
				id: progressBar
				from: 0
				to: 100
				value: vars.progress
				Layout.fillWidth: true
				ToolTip.text: "Simulation progress"
				ToolTip.visible: hovered
			}
			Slider {
				id: speedSlider
				from: 0.0
				to: 2.0
				value: vars.delay
				stepSize: 0.1
				snapMode: Slider.SnapAlways
				ToolTip.text: "Time delay between updates"
				ToolTip.visible: hovered
				onValueChanged: vars.delay = value
			}
		}
	}
}
